/*-
 * Copyright (C)2005 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2005 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: create.c,v 1.5 2010/09/29 16:01:22 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sysexits.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include "multilar.h"
#include "mular_int.h"

mular_descriptor *
mular_create(u_int32_t flags, int lemax, size_t unit, const size_t *size) {
    mular_descriptor *mular;
    size_t mds;

    ifBLIN_QV3(flags)
        fprintf(stderr, "+mular_create flags=%08X lemax=%d unit=%u\n", flags, lemax, (u_int32_t)unit);
    if  (lemax < 1 || lemax >= 31) {
        errno = EINVAL;
        ifBLIN_QV1(flags) warn("lemax: %d", lemax);
        return(NULL);
    }
    mds = (lemax - 1) * sizeof(size_t) + sizeof(mular_descriptor);
    if  (!(mular = malloc(mds))) {
        errno = ENOMEM;
        ifBLIN_QV1(flags) warn("mular_create #1");
        return(NULL);
    }
    bzero(mular, mds);
    mular->flags = flags;
    mular->lemax = lemax;
    if  (unit) {
        mular->unit = unit;
    } else {
        mular->unit = sizeof(void*);
        mular->flags |= MULAR_PFRE | MULAR_SWP1 | MULAR_SWP2;
    }
    if  (mular->flags & (MULAR_CHAR | MULAR_PDMP | MULAR_PFRE) && mular->unit < sizeof(void*)) {
        errno = EINVAL;
        ifBLIN_QV1(flags) warn("No space for pointer in unit");
        return(NULL);
    }
    if  (size) memmove(&mular->size, size, lemax * sizeof(size_t));
    ifBLIN_QV3(flags) fprintf( stderr, "-mular_create flags=%08X lemax=%d unit=%u size=%u\n"
                             , mular->flags, mular->lemax, (u_int32_t)mular->unit, (u_int32_t)mds
                             );
    return(mular);
}
